let patches = ref []
let add_patch patch = patches := patch :: !patches
let filesdir = ref ""
let nosquash = ref false
let refetch = ref false
let camlsyntax = ref false
let version = ref false
let set_quiet () = Message.set_level Message.Warning
let set_veryquiet () = Message.set_level Message.Error
let help () = ()

let usage_msg = "Usage: godiva [options] spec"

let speclist = [
  ("-localbase", Arg.String Config.set_localbase,
              "<dir> specifies a GODI localbase instead of $GODI_LOCALBASE");
  ("-patch", Arg.String add_patch,
          "<patch>   includes the given patch");
  ("-filesdir", Arg.Set_string filesdir,
             "<dir>  includes the given hierarchy of files");
  ("-camlsyntax", Arg.Set camlsyntax,
               "     use the ocaml syntax (devel only)");
  ("-refetch", Arg.Set refetch,
            "        refetches the distfile instead of using a cached one");
  ("-nosquash", Arg.Set nosquash,
             "       fail if an old package is in the way");
  ("-quiet", Arg.Unit set_quiet,
          "          do not print informational messages");
  ("-veryquiet", Arg.Unit set_veryquiet,
              "      do not print warnings or informational messages");
  ("-version", Arg.Set version,
            "        display version information and exit");
  ("-help", Arg.Unit help,
         "           display this help information and exit");
  ("--help", Arg.Unit help,
          "          display this help information and exit");
]

(* some more command line argument state *)
let anonargs = ref []
let anonfun arg = anonargs := arg :: !anonargs


let version () =
    Message.info ("This is GODIVA version " ^ Version.v ^ ".");
    if !version then exit 0 else ()

let run spec_file =
    Misc.sanity_check_localbase (Config.get_localbase ());
    let godi_package =
        if !camlsyntax then
          begin
            Message.info "Using O'Caml syntax for spec file";
            let tmpfile = Filename.temp_file "godiva-mlspec" ".ml" in
            Util.cat_to_file [
                Util.String "open IL\nlet spec = {\n";
                Util.File spec_file;
                Util.String "\n";
                Util.String "}\nlet () = Dynload.spec := Some spec\n";
            ] tmpfile;
            let command = "ocamlfind ocamlc -package godiva -c " ^ tmpfile in
            Message.info command;
            (if Sys.command command <> 0 then
              begin
                Message.error ("Unable to compile the spec file for "
                             ^ "dynamic loading.");
                Message.error ("The file to be compiled is in " ^ tmpfile ^ ".");
                exit 1
              end);
            (* basename really could have the directory in front here: *)
            let dotcmo = (Filename.chop_extension tmpfile) ^ ".cmo" in
            let dotcmi = (Filename.chop_extension tmpfile) ^ ".cmi" in
            (* lest we forget.. *)
            let cleanup () = List.iter Util.remove_if_exists
                                       [dotcmo; dotcmi; tmpfile] in
            (try
                Dynlink.init ();
                (* this sets Dynload.spec side effectually *)
                Dynlink.loadfile dotcmo;
                cleanup ()
            with Dynlink.Error e ->
                Message.error (Dynlink.error_message e);
                cleanup ();
                exit 1);
            (match !Dynload.spec with
              None -> assert false
            | Some spec ->
                Translate.f spec)
          end
        else (* normal spec file syntax *)
          begin
            let lexbuf = Lexer.open_lex spec_file in
            Message.info "parsing spec file..";
            let el = Syntax.handle (fun () -> Parser.spec Lexer.token lexbuf) in
            Message.info "performing semantic analysis..";
            let filesdir = match !filesdir with "" -> None | fd -> Some fd in
            let il = Semanticanalyzer.f ~el ~patches:!patches ~filesdir in
            if !refetch then
              begin
                Message.info "deleting distfile to force refetch..";
                Util.remove_if_exists
                   ((Config.get_localbase ()) ^ "/build/distfiles/"
                  ^ il.IL.sources_basename ^ il.IL.sources_extension)
              end;
            Message.info "translating to godi package..";
            Translate.f il
          end
    in
    Message.info "outputting..";
    Output.f ~godi_package ~nosquash:!nosquash

let () =
    Arg.parse speclist anonfun usage_msg;
    version ();
    match (List.rev !anonargs) with
      [ spec_file ] ->
        (if Config.get_localbase () = "" then
            try
                Config.set_localbase (Sys.getenv "GODI_LOCALBASE")
            with
                Not_found ->
                    Message.error ("No GODI_LOCALBASE specified!  "
                                ^ "Please use the `-localbase' option");
                    Message.error ("or set the GODI_LOCALBASE environment "
                                ^ "variable.");
                    exit 1);
        run spec_file
    | _ -> Arg.usage speclist usage_msg
